/* Copyright (C) 2020-2021 BaseALT Ltd
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <QCommandLineParser>
#include <QTranslator>
#include <QLibraryInfo>

#include "mainwindow.h"
#include "headless.h"

int main(int argc,char **argv)
{

    QApplication app(argc, argv);
    app.setWindowIcon(QIcon::fromTheme(QStringLiteral("custom-vendor-logo"), QIcon::fromTheme(QStringLiteral("basealt"))));

    QString locale = qEnvironmentVariable("LANG");
    QTranslator *translator = new QTranslator(&app);
    translator->load("alt_csp_cryptopro_" + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    QApplication::instance()->installTranslator(translator);
    
    QTranslator *systemTranslator = new QTranslator(&app);
    systemTranslator->load("qt_" + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    QApplication::instance()->installTranslator(systemTranslator);

    QCommandLineParser *parser = new QCommandLineParser();
    QCommandLineOption verify("verify", QTranslator::tr("Verify given file and signature."));
    parser->addOption(verify);
    parser->addPositionalArgument("file", QTranslator::tr("The file to verify or sign."));
    parser->addPositionalArgument("signature",
                                  QTranslator::tr("the signature to verify."),
                                  QTranslator::tr("[signature]"));
    parser->addHelpOption();
    parser->process(app);

    bool isVerify = parser->isSet(verify);

    MainWindow mw;
    if (isVerify) {
        Headless hs(&mw);
        return hs.verify(parser->positionalArguments());
    } else {
        mw.show();
    }
    mw.init(parser->positionalArguments());
    return app.exec();
}
