/* Copyright (C) 2020-2021 BaseALT Ltd
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QTableView>

#include "cache.h"
#include "process.h"
#include "dialog.h"
#include "zip.h"

#include "ui_mainwindow.h"

enum Status
{
    Normal,
    Error,
};

enum PendingAction
{
    Noop = 0,
    Sign,
    Verify
};

namespace Ui {
class MainWindow;
}

class MainWindow : public QWidget
{
    Q_OBJECT

    friend QTableView;
public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

    void init(QStringList);

private:
    Ui::MainWindow *ui;
    QString m_archivePath;
    QString m_documentPath;
    QString m_signaturePath;
    QScopedPointer<Cache> m_cache;
    QScopedPointer<Dialog> m_dialog;
    QTimer m_waitTimer;
    QString m_dir;
    QStringList m_extensionsAllowed;
    PendingAction m_pending;
    bool m_compress;
    QString m_pendingSignature;
    QStringList m_fprintsPending;
    QStringList m_signaturesPending;

    void loadStyleSheet();
    QString getDocumentDir();
    QString getSignatureDir();
    void saveDir();
    void addExtension(QString extension);

    void loadCerts();
    QStringList currentExtensions();

    QStringList changeArchive(QString path);
    void changeDocument(QString path);

    void generateName();

    QString generateSignatureName(QString);
    void updateSignatureExtension();
    void changeSignatureFile(QString);
    void compressSignature(QString, QStringList);

    void viewDocument();
    void selectDocument();
    void selectSignature();
    void verifySignature();
    void sign();
    QStringList getTableData(int column);

    void enableControls();
    void disableControls();
    void validateSignatureActions();
    bool isValid();

    void startWaiting(Dialog *dialog);
    void stopWaiting();

    void error(QString title, QString error,
                   QString details = QString(), bool stop_waiting = true);
    void showDialog();
    void closeDialog();
    void rmAbortedSignature(QString path);
    void showResult(QString result, Status status);

    void onVerifyerCompleted(Result *result);
    void onSignerCompleted(Result *result);
    bool processCompleted(QObject *p, Result *result);

    bool checkCurrentDocumentExists();
};

#endif //MAINWINDOW_H
