/* Copyright (C) 2020-2021 BaseALT Ltd
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef PROCESS_H
#define PROCESS_H

#include <QProcess>
#include <QTimer>

#include "paths.h"

class Result
{
public:
    Result(QProcess::ProcessError error);
    Result(QProcess::ExitStatus exitStatus,
                    int exitCode);

    QProcess::ProcessError error();
    QProcess::ExitStatus exitStatus();

private:
    QProcess::ProcessError m_error;
    QProcess::ExitStatus m_exitStatus;
    int m_exitCode;
};

class Process : public QProcess
{
    Q_OBJECT
public:
    Process();

    void setKillTime(int killTime);
    bool errorOccured();
    QByteArray standardOutput();
    QByteArray standardError();
    void readStandardError();
    QString command();
    QList<Qt::Key> questionKeys(QByteArray array);

    void verify(const QString filePath, const QString sigPath);
    void sign(QString path, QString sig, QString fprint,
              QString derFormat, bool attached);
    bool findContainers();
    bool getCertificates();

signals:
    void completed(Result *result);

private:
    Result *m_completedResult;
    QTimer m_timer;
    QByteArray m_standardOutput;
    QByteArray m_standardError;

    int killTime();
    QString errorCode();
    void readStandardOutput();
    void onReadyReadStandardOutput();
    void onReadyReadStandardError();
    void onErrorOccured(ProcessError error);
    void onStateChanged(ProcessState state);
    void onFinished(int exitCode, ExitStatus exitStatus);
    void onStarting();
};

#endif // PROCESS_H
